"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeComponentI18nMessages = exports.SUPPORTED_CLIENT_LOCALES = void 0;
const path_1 = __importDefault(require("path"));
const utils_1 = require("../../../common/utils");
exports.SUPPORTED_CLIENT_LOCALES = [
    { name: 'en', strict: true },
    { name: 'ru', strict: true },
    { name: 'fr', strict: false },
];
/**
 * Adds component i18n messages to the frontend client i18n message packs.
 * Also adds a menu item caption to the i18n file for locale `en`. The caption is constructed from the class name.
 * If any message already exists in the file - it will NOT be overwritten with a new value.
 *
 * @deprecated moved to building-blocks
 *
 * @param fs - yeoman fs
 * @param className - component class name
 * @param dirShift - directory depth from project root
 * @param projectLocales - locales enabled for this project.
 * If provided, i18n messages will be added only for these locales.
 * Otherwise, i18n messages for all locales supported by Frontend UI will be added
 * (this situation is possible if the project model was created using an older Studio version and does not
 * contain locales info).
 * @param componentMessagesPerLocale - an object where keys are locale codes (such as 'en' or 'ru) and values
 * are objects containing i18n key/value pairs for that locale.
 */
function writeComponentI18nMessages(fs, className, dirShift = './', projectLocales, componentMessagesPerLocale = { en: {}, ru: {} }) {
    Object.entries(componentMessagesPerLocale).forEach(([localeCode, componentMessages]) => {
        if (projectLocales == null || projectLocales.some(projectLocale => projectLocale.code === localeCode)) {
            const existingMessagesPath = path_1.default.join(dirShift, `i18n/${localeCode}.json`);
            const existingMessages = fs.readJSON(existingMessagesPath);
            const mergedMessages = mergeI18nMessages(existingMessages, componentMessages, className, localeCode);
            if (mergedMessages != null) {
                fs.writeJSON(existingMessagesPath, mergedMessages);
            }
        }
    });
}
exports.writeComponentI18nMessages = writeComponentI18nMessages;
/**
 *
 * @param existingMessages - messages that already exist in the i18n file
 * @param componentMessages - messages required by the component
 * @param className - component class name, menu caption will be generated based on it
 * @param localeCode - e.g. 'en' or 'ru'
 *
 * @return messages to be written to the i18n file or `null` if no messages are to be added.
 * Messages to be added are determined as messages in `componentMessages` that are not
 * present in `existingMessages` plus (for `en` locale only) the menu caption
 * if not already present in `existingMessages`.
 */
function mergeI18nMessages(existingMessages, componentMessages, className, localeCode) {
    const screenCaption = utils_1.splitByCapitalLetter(utils_1.capitalizeFirst(className));
    if (localeCode === 'en') {
        componentMessages = Object.assign(Object.assign({}, componentMessages), { [`screen.${className}`]: screenCaption });
    }
    return hasNewEntries(componentMessages, existingMessages)
        ? Object.assign(Object.assign({}, componentMessages), existingMessages) : null;
}
function hasNewEntries(newVals, oldVals) {
    const newKeys = Object.keys(newVals);
    if (newKeys.length === 0) {
        return false;
    }
    if (!oldVals) {
        return true;
    }
    return !newKeys.every((newK) => Object.keys(oldVals).some((oldK) => oldK === newK));
}
//# sourceMappingURL=i18n.js.map